#!/usr/share/ucs-test/runner pytest-3 -s -l -v
## -*- coding: utf-8 -*-
## desc: Check test app login with saml, get access token and verify self disclosure user metadata
## tags: [ucsschool-id-broker]
## packages: [id-broker-provisioning-api-plugin]
## exposure: safe


import os

import pytest
import requests
from utils import (
    get_self_disclosure_group_users,
    get_self_disclosure_metadata,
    get_self_disclosure_user_groups,
    ucs_saml_login,
    univention_test_app_login_with_saml,
)

from univention.testing.utils import retry_on_error


@pytest.mark.parametrize(
    "uid",
    [
        "Traeger1-stud1",
        "Traeger1-teach1",
        "Traeger2-stud1",
        "Traeger2-teach1",
    ],
)
def test_login_with_saml_assertion_metadata(udm, uid):
    traeger = uid.split("-")[0]
    username = uid.split("-", 1)[1]
    kdc_idp_hint = traeger.lower()
    traeger_fqdn = os.environ.get(f"UCS_ENV_{traeger.upper()}_FQDN")
    test_app_fqdn = os.environ.get("UCS_ENV_TEST_APP_FQDN")
    assert traeger_fqdn
    assert test_app_fqdn
    password = "univention"
    saml_cookies = ucs_saml_login(username, password, traeger_fqdn)
    access_token = univention_test_app_login_with_saml(kdc_idp_hint, test_app_fqdn, saml_cookies)
    user_metadata = get_self_disclosure_metadata(access_token)
    udm_mod = udm.get("users/user")
    broker_user = list(udm_mod.search(f"uid={uid}"))[0]
    # check self disclosure metadata and broker user
    assert user_metadata["username"] == "{}.{}".format(
        broker_user.props.firstname, broker_user.props.lastname
    )
    assert user_metadata["firstname"] == broker_user.props.firstname
    assert user_metadata["lastname"] == broker_user.props.lastname
    assert user_metadata["school_id"] in [
        school.replace(f"{traeger}-", "") for school in broker_user.props.school
    ]
    assert user_metadata["school_authority"] == broker_user.props.ucsschoolSourceUID
    assert user_metadata["type"] in [role.split(":")[0] for role in broker_user.props.ucsschoolRole]


@pytest.mark.parametrize(
    "uid",
    [
        "Traeger1-stud1",
        "Traeger1-teach1",
        "Traeger2-stud1",
        "Traeger2-teach1",
    ],
)
def test_login_with_saml_assertion_user_groups(udm, uid):
    traeger = uid.split("-")[0]
    username = uid.split("-", 1)[1]
    kdc_idp_hint = traeger.lower()
    traeger_fqdn = os.environ.get(f"UCS_ENV_{traeger.upper()}_FQDN")
    test_app_fqdn = os.environ.get("UCS_ENV_TEST_APP_FQDN")
    assert traeger_fqdn
    assert test_app_fqdn
    password = "univention"
    saml_cookies = ucs_saml_login(username, password, traeger_fqdn)
    access_token = univention_test_app_login_with_saml(kdc_idp_hint, test_app_fqdn, saml_cookies)
    user_groups = retry_on_error(lambda: get_self_disclosure_user_groups(access_token))
    udm_mod = udm.get("users/user")
    broker_user = list(udm_mod.search(f"uid={uid}"))[0]
    # check self disclosure user groups

    for group in user_groups["groups"]:
        cn = f"cn={group['school_id']}-{group['name']}"
        assert any(_group.startswith(cn) for _group in broker_user.props.groups)
        dn = [_group for _group in broker_user.props.groups if _group.startswith(cn)][0]
        if "cn=klassen" in dn:
            assert group["type"] == "school_class"
        else:
            assert group["type"] == "workgroup"
        if "stud" in uid:
            with pytest.raises(requests.exceptions.HTTPError) as exc:
                group_users = get_self_disclosure_group_users(access_token, group["group_id"])
                assert exc.status_code == 403
        else:
            group_users = get_self_disclosure_group_users(access_token, group["group_id"])
            assert "students" in group_users
            assert "teachers" in group_users
